@@ -2,7 +2,7 @@ import enum
 import inspect
 from dataclasses import fields as dataclass_fields, is_dataclass
 from functools import partial, wraps
-from typing import Any, Callable, List, Optional
+from typing import List, Optional
 
 import docstring_parser
 
@@ -24,9 +24,9 @@ from graphql.type import (
     GraphQLType,
 )
 
-from pytypes import get_arg_for_TypeVar
+from typed_graphql.util import get_arg_for_typevar
 
-from typing_inspect import is_new_type, is_typevar
+from typing_inspect import is_new_type, is_optional_type, is_typevar
 
 
 RESERVED_ARGUMENT_NAMES = set(["data", "info", "return"])
@@ -276,14 +276,14 @@ def python_type_to_graphql_type(cls, t, nonnull=True, input_field=False):
         return _t
     elif str(t).startswith("typing.List"):
         assert len(t.__args__) == 1
-        _t = GraphQLList(python_type_to_graphql_type(cls, t.__args__[0], nonnull=True))
+        _t = GraphQLList(python_type_to_graphql_type(cls, t.__args__[0], nonnull=True, input_field=input_field))
         if nonnull:
             return GraphQLNonNull(_t)
         return _t
     elif str(t).startswith("typing.Tuple"):
         if not len(set(t.__args__)) == 1:
             raise Exception("tuples must have the same type for all members")
-        _t = GraphQLList(python_type_to_graphql_type(cls, t.__args__[0], nonnull=True))
+        _t = GraphQLList(python_type_to_graphql_type(cls, t.__args__[0], nonnull=True, input_field=input_field))
         if nonnull:
             return GraphQLNonNull(_t)
         return _t
@@ -292,10 +292,10 @@ def python_type_to_graphql_type(cls, t, nonnull=True, input_field=False):
         return GraphQLList(
             python_type_to_graphql_type(cls, t.__args__[0], nonnull=True)
         )
-    elif str(t).startswith("typing.Union") or str(t).startswith("typing.Optional"):
+    elif is_optional_type(t):
         if len(t.__args__) == 2:
             if issubclass(t.__args__[1], type(None)):
-                return python_type_to_graphql_type(cls, t.__args__[0], nonnull=False)
+                return python_type_to_graphql_type(cls, t.__args__[0], nonnull=False, input_field=input_field)
             else:
                 raise Exception
         else:
@@ -303,7 +303,7 @@ def python_type_to_graphql_type(cls, t, nonnull=True, input_field=False):
 
     elif is_new_type(t):
         return python_type_to_graphql_type(
-            cls, t.__supertype__, input_field=input_field
+            cls, t.__supertype__, input_field=input_field, nonnull=nonnull,
         )
 
     elif is_dataclass(t):
@@ -318,8 +318,9 @@ def python_type_to_graphql_type(cls, t, nonnull=True, input_field=False):
         return t
 
     elif is_typevar(t):
+        t = get_arg_for_typevar(t, cls)
         return python_type_to_graphql_type(
-            cls, get_arg_for_TypeVar(t, cls), nonnull=nonnull, input_field=input_field
+            cls, t, nonnull=nonnull, input_field=input_field
         )
 
     else:
@@ -353,6 +354,15 @@ def python_type_to_graphql_type(cls, t, nonnull=True, input_field=False):
                     return GraphQLNonNull(graphql_type(t, input_field=input_field))
                 return graphql_type(t, input_field=input_field)
 
+            elif issubclass(t, inspect._empty):
+                raise PythonToGraphQLTypeConversionException(t)
+
+            # A simple class
+            elif issubclass(t, object):
+                if nonnull:
+                    return GraphQLNonNull(graphql_type(t, input_field=input_field))
+                return graphql_type(t, input_field=input_field)
+
             else:
                 raise PythonToGraphQLTypeConversionException(t)
 
